import React from 'react';
import Cell from './Cell';
import Controls from './Controls';
import { countLivingNeighbors } from './lib';

interface Props {
  width?: number;
  height?: number;
  speed?: number;
}

interface State {
  grid: boolean[][];
  running: boolean;
  speed: number;
}

class Board extends React.Component<Props, State> {
  timer: NodeJS.Timeout | null;

  constructor(props: Props) {
    super(props);
    const height = props.height ?? 10;
    const width = props.height ?? 20;
    this.state = {
      grid: Array(height).fill(Array(width).fill(false)),
      running: false,
      speed: 500,
    };
    this.timer = null;
  }

  start = () => {
    this.timer = setInterval(() => {
      console.log('Iterating');
      const { grid } = this.state;
      const newGrid = [...grid].map((row, y) => {
        const newRow = [...row].map((cell, x) => {
          const isAlive = cell;
          const livingNeighbors = countLivingNeighbors(grid, x, y);

          if (isAlive) {
            if (livingNeighbors === 2 || livingNeighbors === 3) {
              return true;
            } else {
              return false;
            }
          } else {
            if (livingNeighbors === 3) {
              return true;
            }
          }

          return isAlive;
        });

        return newRow;
      });

      this.setState({ grid: newGrid });
    }, this.state.speed);
  };

  stop = () => {
    if (this.timer) clearInterval(this.timer);
  };

  toggleCell = (x: number, y: number) => {
    const newGrid = [...this.state.grid];
    const newRow = [...newGrid[y]];
    newRow[x] = !newGrid[y][x];
    newGrid[y] = newRow;
    this.setState({ grid: newGrid });
  };

  toggleRunning = () => {
    if (this.state.running) this.stop();
    if (!this.state.running) this.start();
    this.setState({ running: !this.state.running });
  };

  updateSpeed = (speed: number) => {
    this.setState({ speed });
    if (this.state.running) {
      this.stop();
      this.start();
    }
  };

  render() {
    const { grid, running, speed } = this.state;

    return (
      <div className="board">
        <Controls
          running={running}
          speed={speed}
          toggle={this.toggleRunning}
          updateSpeed={this.updateSpeed}
        />
        {grid.map((row, y) => (
          <div className="row" key={y}>
            {row.map((cell, x) => (
              <Cell
                disabled={running}
                x={x}
                y={y}
                alive={cell}
                toggle={this.toggleCell}
                key={x}
              />
            ))}
          </div>
        ))}
      </div>
    );
  }
}

export default Board;
